/*
m_loop_new - Record a sample (for looping)

DESCRIPTION
m_loop_new records an audio signal to a table. It has an optional offset
before recording begins and an optionsl recording duration, which defaults to
the running time of the instrument/current note.

SYNTAX
kstatus m_loop_new Sname ainput [, itime] [, ioffset]

INITIALIZATION
itime - Duration of the recording (default=p3)
ioffset - Time offset in seconds before recording starts (default=0)

PERFORMANCE
kstatus - Output status: 0=recording/waiting, 1=finished
ainput - Input audio signal to sample

CREDITS
Author: Jeanette C.
*/

opcode m_loop_new, k, Saoo
	Sname, aInput, iTime, iOffset xin

	; Initialise the status
	kStatus init 0 ; not finished yet
	kStart init 0 ; I/O param for tablewa

	; Set the basic channel name
	SchannelName strcat "m_loop.", Sname
	chn_k strcat(SchannelName, ".Valid"), 3

	; Set the actual recording length, p3 if nothing specified
	if iTime == 0 then
		iDuration init p3
	else
		iDuration init iTime
	endif

	if (iOffset + iDuration) > p3 then
		prints "\nWARNING m_loop_new %s: note duration (p3) too short to capture full loop.", Sname
	endif

	iLength = ceil(iDuration * sr)
	iEnd init iLength

	; Create the table to hold our sample
	iRecorder ftgentmp 0, 0, -(iLength + ksmps), -2, 0 ; be sure, extend by ksmps
	iSample ftgen 0, 0, -iLength, -2, 0
	chnset iSample, strcat(SchannelName, ".Table")

	ktime timeinsts ; get the current time

	if (ktime >= iOffset) && (kStart <= iEnd) then
		; begin recording
		kStart tablewa iSample, aInput, 0
	elseif (kStart > iEnd) then ; recording done
		kStatus = 1
		chnset 1, strcat(SchannelName, ".Valid") ; this loop is now valid to play
	endif

	; output the status
	xout kStatus
endop
